/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa;

import com.pagosoft.plaf.PgsLookAndFeel;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import monalisa.gui.MainDialog;

/**
 * Application entry point class for MonaLisa.
 * 
 * @author Konrad Rudolph and Jens Einloft
 */
public final class MonaLisa implements Runnable {
    private static final String OS_NAME = System.getProperty("os.name");
    public static final boolean IS_MACOSX = OS_NAME.toLowerCase().startsWith("mac os x");
    public static final String APPLICATION_TITLE = "MonaLisa";

    private static MainDialog applicationMainWindow;
    
    /**
     * Returns a reference to the application's main window.
     */
    public static MainDialog appMainWindow() {
        return applicationMainWindow;
    }
    
    private MonaLisa() { }

    /**
     * Application entry point
     * 
     * @param args
     * @throws UnsupportedLookAndFeelException
     * @throws IllegalAccessException
     * @throws InstantiationException
     * @throws ClassNotFoundException
     */
    public static void main(String[] args) throws ClassNotFoundException, InstantiationException, IllegalAccessException, UnsupportedLookAndFeelException, FileNotFoundException {
        try {
            System.setErr(new PrintStream(new FileOutputStream(System.getProperty("user.home")+"/MonaLisaErrorLog.txt")));
        } catch (FileNotFoundException ex) {
            
        }
        if (IS_MACOSX) {
            // Set system properties for correct styling under OS X.
            System.setProperty("apple.laf.useScreenMenuBar", "true");
            System.setProperty("com.apple.mrj.application.apple.menu.about.name", APPLICATION_TITLE);
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());                       
        }

        UIManager.setLookAndFeel(new PgsLookAndFeel());
        SwingUtilities.invokeLater(new MonaLisa());
        
        // First start of MonaLisa? Than write the config file for the NetViewer Settings
        if(!Settings.load()) 
            Settings.createDefaultConfigFile();           
    }

    @Override
    public void run() {
        applicationMainWindow = new MainDialog();
    }
}
