/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.data.pn;

/**
 * A Petri net entity that is identifiable by some unique identifier.
 * 
 * Unique entities may be used meaningfully in equality comparisons, and as
 * keys in mapping relations.
 * Notice that the object <em>must</em> have a unique identifier, or else using
 * it may result in undefined behavior.
 * @author Konrad Rudolph
 */
public abstract class UniquePetriNetEntity extends AbstractPetriNetEntity {
    private static final long serialVersionUID = 2189444289799672226L;

    protected UniquePetriNetEntity(int id) {
        putProperty("id", id);
    }
    
    protected UniquePetriNetEntity(UniquePetriNetEntity other) {
        super(other);
        putProperty("id", other.id());
    }
    
    public int id() {
        return ((Integer)getProperty("id")).intValue();
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj == this)
            return true;
        else if (obj.getClass() == getClass())
            return id() == ((UniquePetriNetEntity) obj).id();
        else
            return false;
    }
    
    @Override
    public int hashCode() {
        return id();
    }
}
