/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.tools.cluster;

import java.io.*;
import java.util.*;

import monalisa.data.pn.TInvariant;
import monalisa.results.DistanceMatrix;

/*
 * Die Klasse Cluster.java stellt die zentrale Klasse der Programmoption
 * Clusteranalyse dar. Sie beinhaltet: - Aufruf und Durchfuehrung der
 * ausgewaehlten Clusteranalyseverfahren - Methode zur grafischen Darstellung
 * des Dendrogramms - Erzeugung der xwg.Datei
 * @author belau
 */
public class Cluster implements Serializable {
    String name; // Clusterlabel
    String member; // Liste der Namen der Clustermitglieder
    String label;
    int index; // Clusterindex
    int cn; // Clusteranzahl innerhalb eines Clusters
    int leftCn; // Clusteranzahl innerhalb des linken Kind-Clusters
    int rigthCn; // Clusteranzahl innerhalb des rechten Kind-Clusters
    float mitte; // Baum: x-Koordinate eines inneren Knotens
    float leaveHeigth; // Abstand des Clusters zu den Blaettern
    float leftHeigth; // Abstand Eltern-Cluster linkes Kind
    float rightHeigth; // Abstand Eltern-Cluster rechtes Kind
    Cluster left; // linkes Kind
    Cluster right; // rechtes Kind
    int leftoc = 0;
    int rightoc = 0;
    int oc = 0;

    static ArrayList<Nodes> node = new ArrayList<Nodes>();
    static ArrayList<Edge> edge = new ArrayList<Edge>();
    static Vector<Leaves> groups = new Vector<Leaves>();

    static int ui = 0;
    static int cIndex = 0;
    
    public Cluster() {
        this(null, null, 0, 0, 0, 0, 0, null, 0, null, 0, 0, 0, 0, 0);
    }

    public Cluster(String member, String name, int index, int cn, int leftCn,
            int rigthCn, float leaveHeigth, Cluster left, float leftHeigth,
            Cluster right, float rightHeigth, float mitte, int oc, int leftoc,
            int rightoc) {
        this.name = name;
        this.member = member;
        this.index = index;
        this.mitte = mitte;
        this.cn = cn;
        this.leftCn = leftCn;
        this.rigthCn = rigthCn;
        this.left = left;
        this.right = right;
        this.leaveHeigth = leaveHeigth;
        this.leftHeigth = leftHeigth;
        this.rightHeigth = rightHeigth;
        this.leftoc = leftoc;
        this.rightoc = rightoc;
        this.oc = oc;

        // Initialisierung der Clusteranzahl cn
        if (left == null && right == null) {
            cn = 1;
        } else {
            cn = leftCn + rigthCn;
        }
        index++;
    }

    public String getName() {
        return name;
    }

    public int getIndex() {
        return index;
    }

    public float getHeigth() {
        return leaveHeigth;
    }

    public int getCn() {
        return cn;
    }

    public float getMitte() {
        return mitte;
    }

    public class Nodes implements Serializable {
        String name;
        String label;

        float x;
        float y;

        public Nodes(String name, String label, float x, float y) {
            this.name = name;
            this.label = label;
            this.x = x;
            this.y = y;
        }

        public String getName() {
            return name;
        }

        public String getLabel() {
            return label;
        }

        public float getX() {
            return x;
        }

        public float getY() {
            return y;
        }

    }

    public class Edge implements Serializable {
        String start;
        String end;

        public Edge(String start, String end) {
            this.start = start;
            this.end = end;
        }

        public String getStart() {
            return start;
        }

        public String getEnd() {
            return end;
        }
    }

    /* Methode zur grafischen Darstellung des Baums */
    public void paint(Cluster urwurzel, int counter, String verfahren, float y,
            int leaves, float heigth, Cluster wurzel, int xPosition,
            float leftPosition, float rightPosition, float mitte, float oben,
            float unten) {

        if (wurzel != null) {
            float distLeft; // absolute y-Koordinate linkes Kind
            float distRigth; // absolute y-Koordinate rechtes Kind
            float xFaktor; // Faktoren zur Berechnung der absoluten
            // x-Koordinaten (Verhaeltnis:
            // Fensterbreite/Clusteranzahl)
            float yFaktor = y; // Faktoren zur Berechnung der absoluten
            // y-Koordinaten (Verhaeltnis:
            // Fensterlaenge/Clusteranzahl/)

            // Initialisierung der x- & y-Faktoren
            xFaktor = 0.2f; // 0.025
            // Initialisierung der absoluten y-Koordinaten der Kinder
            distRigth = wurzel.rightHeigth * yFaktor;
            distLeft = wurzel.leftHeigth * yFaktor;

            // Berechnung der x-Koordinaten
            if (wurzel.left != null && wurzel.right != null) { // innere Knoten
                leftPosition = wurzel.left.mitte;
                rightPosition = wurzel.right.mitte;
                mitte = (leftPosition + rightPosition) / 2;
            } else {
                mitte = (xPosition + 0.498765f) * xFaktor; // Blaetterknoten
                leftPosition = mitte;
                rightPosition = mitte;
                if (counter != 1) {
                    cIndex = 0;
                }
                if (counter == 1) {
                    ++cIndex;
                    String label = "Cluster" + cIndex;
                    node.add(new Nodes(wurzel.name, label, mitte, oben));
                    groups.add(new Leaves(wurzel.name, wurzel.member, label, 0,
                            new Vector<String>(), new Vector<Integer>()));

                }
            }
            // Zuweisung der berechneten x-Koordinate des inneren Knotens
            // (mitte)
            wurzel.mitte = mitte;

            // linken Unterbaum zeichnen
            paint(urwurzel, counter, verfahren, y, leaves, heigth, wurzel.left,
                    xPosition, leftPosition, rightPosition, mitte, oben
                            + distLeft, unten);
            // rechten Unterbaum zeichnen
            paint(urwurzel, counter, verfahren, y, leaves, heigth,
                    wurzel.right, (xPosition + wurzel.leftCn), leftPosition,
                    rightPosition, mitte, oben + distRigth, unten);

            if (counter == 1) {
                if (wurzel.left != null) {
                    node.add(new Nodes(wurzel.name, "", mitte, oben));
                    String name = "N" + (++ui);
                    node.add(new Nodes(name, "", leftPosition, oben));
                    edge.add(new Edge(wurzel.name, name));
                    edge.add(new Edge(name, wurzel.left.name));
                }
                if (wurzel.right != null) {
                    String name = "N" + (++ui);
                    node.add(new Nodes(name, "", rightPosition, oben));
                    edge.add(new Edge(wurzel.name, name));
                    edge.add(new Edge(name, wurzel.right.name));
                }
            }
        }// if
    }// paint

    public boolean generateFile(
            List<TInvariant> allTinvariants,
            List<TInvariant> filteredTinvariants,
            DistanceMatrix matrix,
            String distanceFunc, 
            String clusteringfunc, 
            String fileNameTarget,
            String outputfile, 
            float threshold) {

        Cluster wurzel = null;
        float[][] d;
        float[][] e;
        String[] label;
        int n; // Invarianten-Anzahl
        String distfunc;
        float th;
        float y;
        n = filteredTinvariants.size();

        try {
            n = filteredTinvariants.size();
            d = new float[n + 1][n + 1];
            e = d;
            /* Indizierung + Einlesen der Distanzmatrix */
            for (int i = 0; i < n + 1; i++) {
                d[i][0] = i;
            }
            for (int j = 0; j < n + 1; j++) {
                d[0][j] = j;
            }
            for (int i = 1; i < n + 1; i++) {
                for (int j = 1; j < n + 1; j++) {
                    d[i][j] = matrix.get(i - 1, j - 1);
                }
            }
            label = new String[n];
            for (int i = 0; i < n; i++) {
                String name = "Inv" + filteredTinvariants.get(i).id();
                label[i] = name;
            }
            distfunc = distanceFunc;
        } catch (java.lang.Exception a) {
            a.printStackTrace();
            return false;
        }

        try {
            node.clear();
            edge.clear();
            groups.clear();
            String verfahren = clusteringfunc;

            // Berechnung des tresholds
            // float th1 = 100 - th;
            th = 100 - threshold;
            if (distfunc.equals("sum_of_differences")) {
                th = th * 60 / 100;
                y = 0.05f;
            } else {
                th = th * 30 / 100;
                y = 0.09f;
            }

            if (verfahren == ClusterFunctions.SingleLinkage) {
                Single u = new Single(d, e, label, th);
                wurzel = u.getWurzel();
            } else if (verfahren == ClusterFunctions.CompleteLinkage) {
                Complete u = new Complete(d, e, label, th);
                wurzel = u.getWurzel();
            } else if (verfahren == ClusterFunctions.UPGMA) {
                UPGMA u = new UPGMA(d, e, label, th);
                wurzel = u.getWurzel();
            } else if (verfahren == ClusterFunctions.WPGMA) {
                WPGMA u = new WPGMA(d, e, label, th);
                wurzel = u.getWurzel();
            } else if (verfahren == ClusterFunctions.NeighborJoining) {
                NJ u = new NJ(d, e, label);
                wurzel = u.getWurzel();
            } else {
                System.out.println("Falsche Eingabe.");
            }

            // Deklaration und Initialisierung der Attribute heigth (Gesamthoehe
            // des Baumes), leaves (Gesamtblaetteranzahl)
            float heigth = (wurzel.getHeigth());
            int leaves = wurzel.getCn();
            Cluster urwurzel = wurzel;
            ui = urwurzel.index;
            float a = 0;
            try{
                a = urwurzel.right.mitte;
            } catch (NullPointerException npe) {
                System.out.println("The treshold-value is to low: all invariants are grouped in one cluster. Please choose a higher value.");
                return false;
            }
            float b = 2.0f;
            int counter = 0;
            int counter2 = 0;
            while (counter2 <= 10) {
                a = b;
                wurzel.paint(urwurzel, counter, verfahren, y, leaves, heigth,
                        wurzel, 0, 0, 0, 0, 6.0f, 0.5f);
                b = urwurzel.right.mitte;
                if (b == a) {
                    counter2++;
                    if (counter2 == 11) {
                        counter = 1;
                        wurzel.paint(urwurzel, counter, verfahren, y, leaves,
                                heigth, wurzel, 0, 0, 0, 0, 6.0f, 0.5f);
                    }
                }
            }

            PrintWriter out = new PrintWriter(new FileWriter(fileNameTarget));
            out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
            out.write("<!DOCTYPE WilmaGraph SYSTEM \"WilmaGraph.dtd\">\n");
            out.write("<WilmaGraph><Cluster><LayoutEngineType Name=\"Force Directed\"></LayoutEngineType>\n");
            for (int i = 0; i < node.size(); i++) {
                Nodes v = (Nodes) node.get(i);
                String z = v.getName();
                if (z.startsWith("I")) {
                    out.write("<Node ID=\"" + v.getName()
                            + "\"><ViewType Name=\"DefaultNodeView\">\n");
                    out
                            .write("<Property Key=\"Colour\" Value=\"0.05084745762711862 1.0 0.0\"/><Property Key=\"Radius\" Value=\"0.050000\"/><Property Key=\"Label\" Value=\""
                                    + v.getLabel() + "\"/></ViewType>\n");
                    double q = v.getY() * 0.999999f;
                    q = Math.round(q * 1000000.) / 1000000.;
                    double p = v.getX() * 0.999999f;
                    p = Math.round(p * 1000000.) / 1000000.;
                    out.write("<Property Key=\"Position\" Value=\"" + q + " "
                            + p + " 0.000000\"/></Node>\n");
                } else {
                    out.write("<Node ID=\"" + v.getName()
                            + "\"><ViewType Name=\"DefaultNodeView\">\n");
                    out
                            .write("<Property Key=\"Colour\" Value=\"0.500000 0.500000 0.500000\"/><Property Key=\"Radius\" Value=\"0.020000\"/></ViewType>\n");
                    double q = v.getY() * 0.999999f;
                    q = Math.round(q * 1000000.) / 1000000.;
                    double p = v.getX() * 0.999999f;
                    p = Math.round(p * 1000000.) / 1000000.;
                    out.write("<Property Key=\"Position\" Value=\"" + q + " "
                            + p + " 0.000000\"/></Node>\n");
                }
            }
            for (int j = 0; j < edge.size(); j++) {
                Edge x = (Edge) edge.get(j);
                out.write("<Edge StartID=\"" + x.getStart() + "\" EndID=\""
                        + x.getEnd() + "\"><ViewType Name=\"Plain Edge\">\n");
                out.write("<Property Key=\"Radius\" Value=\"0.020000\"/><Property Key=\"Colour\" Value=\"0.500000 0.500000 0.500000\"/></ViewType></Edge>\n");
            }
            out.write("</Cluster></WilmaGraph>\n");
            out.close();
            
        } catch (IOException b) {
            b.printStackTrace();
            System.out.println("Error occured while generating the Wilma dendrogram");
            return false;
        } catch (java.lang.Exception g) {
            g.printStackTrace();
            System.out.println("The treshold-value is to low: all invariants are grouped in one cluster. Please choose a higher value.");
            return false;
        }

        try {
            // Erzeugung der output-Datei
            Group g = new Group();
            g.generateGroupFile(e, label, allTinvariants, filteredTinvariants, groups, outputfile);
            
        } catch (java.lang.Exception f) {
            f.printStackTrace();
            System.out.println("File-Error occured: Please check if the distance matrix is based on the source files (*.spped; *.inv).");
            return false;
        }
        return true;
    }

    public ArrayList<Nodes> getNodes() {
        return node;
    }

    public ArrayList<Edge> getEdges() {
        return edge;
    }

    public Vector<Leaves> getGroups() {
        return groups;
    }
}