/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.tools.cluster;

/*
 * Die Klasse Single implementiert das hierarchische agglomerative
 * Clusterverfahren SINGLE-LINKAGE
 */
public class Single extends UPGMA {

    /* Konstruktor der Klasse Single */
    public Single(float[][] d, float[][] e, String[] label, float th) {
        super(d, e, label, th);
    }

    /* Methode zur Neuberechnung der Distanzmatrix */
    public void update(float[][] d, float[][] e) {

        if (imin < jmin) { // Voraussetzung für update: imin < jmin
            // EMPTY!
        } else {
            int i = imin;
            imin = jmin;
            jmin = i;
        }

        for (int i = 1; i <= d.length - 1; i++) {
            /*
             * Berechnung der Distanz zwischen dem neuen Cluster k und allen
             * anderen Clustern
             */
            if ((i != imin) && (i != jmin) && (d[imin][i] != 0)) {
                float betrag = d[imin][i] - d[jmin][i];
                if (betrag < 0) {
                    betrag *= (-1);
                }
                e[imin][i] = (d[imin][i] + d[jmin][i] - betrag) / 2;
            } else {
                e[imin][i] = 1000f;
            }
        }

        /* Kopieren von e in d & Symmetrierung der Distanzmatrix */
        for (int i = 1; i <= d.length - 1; i++) {
            d[i][imin] = e[imin][i];
            d[imin][i] = d[i][imin];
        }

        /* Loeschen der Reihe und Spalte von jmin */
        for (int i = 1; i <= d.length - 1; i++) {
            d[i][jmin] = 10000f;
            d[jmin][i] = d[i][jmin];
        }

        /* Update der Beschriftung */
        d[imin][0] = n;
        d[0][imin] = n;
    }// update
}// class Single