/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.tools.knockout;

import monalisa.data.pn.PetriNet;
import monalisa.data.pn.UniquePetriNetEntity;

import monalisa.tools.ErrorLog;
import monalisa.tools.ProgressEvent;
import monalisa.tools.ProgressListener;
import monalisa.tools.tinv.TInvariantCalculationFailedException;
import monalisa.tools.tinv.TInvariantCalculator;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import monalisa.data.pn.TInvariant;
import monalisa.data.pn.Transition;

public abstract class KnockoutAlgorithm {
    
    private PetriNet pn;
    private Map<List<String>, List<String>> results = new HashMap<List<String>, List<String>>();
    
    KnockoutAlgorithm(PetriNet pn) {
        this.pn = pn;
    }
    
    public void run(ProgressListener callback, ErrorLog log) throws InterruptedException, TInvariantCalculationFailedException {
        TInvariantCalculator calculator = null;
        
        int instance = 0;
        List<String> originalTransitions = new ArrayList<String>();
        Map<String, Integer> tmpMap;
        List<String> knockedOut;
        List<String> alsoKnockedOut;

        for(Transition t : pn.transitions())
            originalTransitions.add((String) t.getProperty("name"));

        // Knock-out
        while(hasNextKnockOutNetwork()) {
            calculator = new TInvariantCalculator(getNextKnockOutNetwork(), log);
            knockedOut = new ArrayList<String>();
            alsoKnockedOut = new ArrayList<String>();
            tmpMap = new HashMap<String, Integer>();
            for(UniquePetriNetEntity e : getKnockoutEntities())
                knockedOut.add((String) e.getProperty("name"));
            for(TInvariant tinv : calculator.tinvariants(log)) {
                for(Transition t : tinv) {
                    if(tinv.factor(t) > 0) {
                        tmpMap.put((String) t.getProperty("name"), 1);
                    }
                }
            }

            for(String name : originalTransitions) {
                if(!tmpMap.containsKey(name) && !knockedOut.contains(name))
                    alsoKnockedOut.add(name);
            }
            results.put(knockedOut, alsoKnockedOut);
            
            int percent = (int) ((double) ++instance / getTotalKnockouts() * 100);
            callback.progressUpdated(new ProgressEvent(this, percent));            
        }
    }
    
    public Map<List<String>, List<String>> getResults() {
        return Collections.unmodifiableMap(results);
    }
    
    protected PetriNet getPetriNet() {
        return pn;
    }
    
    protected abstract int getTotalKnockouts();
    
    protected abstract PetriNet getNextKnockOutNetwork();
    
    protected abstract boolean hasNextKnockOutNetwork();
    
    protected abstract List<UniquePetriNetEntity> getKnockoutEntities();
}
