/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.tools.knockout;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

import monalisa.data.pn.PetriNet;
import monalisa.data.pn.Place;
import monalisa.data.pn.Transition;
import monalisa.data.pn.UniquePetriNetEntity;

public class SinglePlaceKnockout extends KnockoutAlgorithm {

    private Iterator<Place> iterator;
    private List<UniquePetriNetEntity> currentKnockouts;
    
    public SinglePlaceKnockout(PetriNet pn){
        super(pn);
        iterator = pn.places().iterator();
    }
    
    public SinglePlaceKnockout(PetriNet pn, List<Place> places) {
        super(pn);
        iterator = places.iterator();
    }
    
    @Override
    protected PetriNet getNextKnockOutNetwork() {
        PetriNet copy = new PetriNet(getPetriNet());
        Place place = iterator.next();
        
        currentKnockouts = new ArrayList<UniquePetriNetEntity>();
        currentKnockouts.add(place);
        
        for(Transition t : place.outputs())
            copy.removeTransition(t);

        for(Transition t : place.inputs())
            copy.removeTransition(t);
        
        copy.removePlace(place);
        return copy;
    }

    @Override
    protected boolean hasNextKnockOutNetwork() {
        return iterator.hasNext();
    }
    
    protected List<UniquePetriNetEntity> getKnockoutEntities() {
        return currentKnockouts;
    }

    @Override
    protected int getTotalKnockouts() {
        return getPetriNet().places().size();
    }
}
